@@ -52,16 +52,19 @@ from shutil import rmtree
 import rpy2.rinterface as ri
 import rpy2.robjects as ro
 import rpy2.robjects.packages as rpacks
-from rpy2.robjects.conversion import singledispatch
+from rpy2.robjects.conversion import make_converter
 
 try:
-    from rpy2.robjects import pandas2ri as py2ri
+    from rpy2.robjects import pandas2ri as baseconversion
+    template_converter = baseconversion.converter
 except ImportError:
     try:
-        from rpy2.robjects import numpy2ri as py2ri
+        from rpy2.robjects import numpy2ri as baseconversion
+        template_converter = baseconversion.converter
     except ImportError:
         # Give up on numerics
-        py2ri = None
+        baseconversion = None
+        template_converter = ro.conversion.converter
 
 
 # IPython imports
@@ -72,9 +75,14 @@ from IPython.core.magic import (Magics, magics_class, line_magic,
 from IPython.core.magic_arguments import (
     argument, magic_arguments, parse_argstring, argument_group
 )
-from IPython.external.simplegeneric import generic
 from IPython.utils.py3compat import str_to_unicode, unicode_to_str, PY3
 
+try:
+    from IPython.external.simplegeneric import generic
+except ImportError:
+    # IPython 4.0
+    from simplegeneric import generic
+
 
 if sys.version_info[0] == 3:
     unicode = str
@@ -107,13 +115,9 @@ def pyconverter(pyobj):
     """
     return pyobj
 
-@singledispatch
-def ipython2ri(obj):
-    return obj
 
-@singledispatch
-def ri2ipython(obj):
-    return obj
+converter = make_converter('ipython conversion',
+                           template = template_converter)
 
 
 # The default conversion for lists is currently to make them an R list. That has
@@ -222,8 +226,8 @@ utils.install_packages('Cairo')
         withVisible is a LISPy R function).
 
         '''
-        old_writeconsole = ri.get_writeconsole()
-        ri.set_writeconsole(self.write_console)
+        old_writeconsole_regular = ri.get_writeconsole_regular()
+        ri.set_writeconsole_regular(self.write_console_regular)
         try:
             # Need the newline in case the last line in code is a comment
             value, visible = ro.r("withVisible({%s\n})" % code)
@@ -231,10 +235,10 @@ utils.install_packages('Cairo')
             warning_or_other_msg = self.flush() # otherwise next return seems to have copy of error
             raise RInterpreterError(code, str_to_unicode(str(exception)), warning_or_other_msg)
         text_output = self.flush()
-        ri.set_writeconsole(old_writeconsole)
+        ri.set_writeconsole_regular(old_writeconsole_regular)
         return text_output, value, visible[0]
 
-    def write_console(self, output):
+    def write_console_regular(self, output):
         '''
         A hook to capture R's stdout in a cache.
         '''
@@ -287,7 +291,7 @@ utils.install_packages('Cairo')
                     # variable
                     raise NameError("name '%s' is not defined" % input)
             robj = self.pyconverter(val)
-            ro.r.assign(input, ipython2ri(robj))
+            ro.r.assign(input, robj)
 
     # @skip_doctest
     @magic_arguments()
@@ -331,7 +335,8 @@ utils.install_packages('Cairo')
         args = parse_argstring(self.Rpull, line)
         outputs = args.outputs
         for output in outputs:
-            self.shell.push({output: ri2ipython(ro.r(output)) })
+            robj = ri.globalenv.get(output)
+            self.shell.push({output: converter.ri2py(robj) })
 
     # @skip_doctest
     @magic_arguments()
@@ -362,8 +367,10 @@ utils.install_packages('Cairo')
         '''
         args = parse_argstring(self.Rget, line)
         output = args.output
-        res = ro.r(output[0])
-        return ri2ipython(res)
+        # get the R object with the given name, starting from globalenv
+        # in the search path
+        res = ri.globalenv.get(output[0])
+        return converter.ri2py(res)
 
 
     def setup_graphics(self, args):
@@ -429,16 +436,18 @@ utils.install_packages('Cairo')
         md = {}
 
         if self.device == 'png':
-            for imgfile in glob("%s/Rplots*png" % graph_dir):
-                if stat(imgfile).st_size >= 1000:                
-                    images.append(open(imgfile, 'rb').read())
+            for imgfile in sorted( glob("%s/Rplots*png" % graph_dir) ):
+                if stat(imgfile).st_size >= 1000:
+                    with open(imgfile, 'rb') as fh_img:
+                        images.append(fh_img.read())
         else:
             # as onefile=TRUE, there is only one .svg file
             imgfile = "%s/Rplot.svg" % graph_dir
             # Cairo creates an SVG file every time R is called
             # -- empty ones are not published
             if stat(imgfile).st_size >= 1000:
-                images.append(open(imgfile, 'rb').read())
+                with open(imgfile, 'rb') as fh_img:
+                    images.append(fh_img.read())
 
         mimetypes = { 'png' : 'image/png', 'svg' : 'image/svg+xml' }
         mime = mimetypes[self.device]
@@ -637,11 +646,11 @@ utils.install_packages('Cairo')
                 text_result, result, visible = self.eval(code)
                 text_output += text_result
                 if visible:
-                    old_writeconsole = ri.get_writeconsole()
-                    ri.set_writeconsole(self.write_console)
+                    old_writeconsole_regular = ri.get_writeconsole_regular()
+                    ri.set_writeconsole_regular(self.write_console_regular)
                     ro.r.show(result)
                     text_output += self.flush()
-                    ri.set_writeconsole(old_writeconsole)
+                    ri.set_writeconsole_regular(old_writeconsole_regular)
 
         except RInterpreterError as e:
             # XXX - Maybe we should make this red or something?
@@ -657,7 +666,6 @@ utils.install_packages('Cairo')
         if text_output:
             # display_data.append(('RMagic.R', {'text/plain':text_output}))
             publish_display_data(data={'text/plain':text_output}, source='RMagic.R')
-
         # publish the R images
         if self.device in ['png', 'svg']:
             display_data, md = self.publish_graphics(tmpd, args.isolate_svgs)
@@ -671,7 +679,8 @@ utils.install_packages('Cairo')
 
         if args.output:
             for output in ','.join(args.output).split(','):
-                self.shell.push({output: ro.r(output) })
+                output_ipy = converter.ri2py(ri.globalenv.get(output))
+                self.shell.push({output: output_ipy })
 
 
         # this will keep a reference to the display_data
@@ -685,7 +694,7 @@ utils.install_packages('Cairo')
         # so return the converted result
         if return_output and not args.noreturn:
             if result is not ri.NULL:
-                return ri2ipython(result)
+                return converter.ri2py(result)
 
 __doc__ = __doc__.format(
                 R_DOC = ' '*8 + RMagics.R.__doc__,
@@ -695,13 +704,46 @@ __doc__ = __doc__.format(
 )
 
 
+_switch_conversion_dataframe_doc = """
+    Switch the conversion of R data.frame / rpy2's DataFrame objects on or off.
+
+    With rpy2 version 2.6.0, the conversion of R's data.frame objects
+    (rpy2's DataFrame) to pandas' DataFrame is automatically activated
+    when rmagic is loaded in ipython.
+
+    This is trading convenience for performances since data are
+    currently copied during the conversion. This function provides a simple
+    way to to turn that automatic conversion off.
+    """
+
+def switchoff_conversion_dataframes():
+    __docstring__ = _switch_conversion_dataframe_doc
+    import rpy2.robjects.vectors
+    import rpy2.rinterface
+    converter.ri2py.register(rpy2.robjects.vectors.DataFrame,
+                             lambda x: x)
+    converter.ri2py.register(rpy2.rinterface.ListSexpVector,
+                             lambda x: x)
+
+def switchon_conversion_dataframes():
+    __docstring__ = _switch_conversion_dataframe_doc
+    import rpy2.robjects.vectors
+    import rpy2.rinterface
+    cls = rpy2.rinterface.ListSexpVector
+    converter.ri2py.register(rpy2.robjects.vectors.DataFrame,
+                             rpy2.robjects.pandas2ri.ri2py.registry[cls])
+    converter.ri2py.register(cls,
+                             rpy2.robjects.pandas2ri.ri2py.registry[cls])
+
+
+
 def load_ipython_extension(ip):
     """Load the extension in IPython."""
 
-    if py2ri is not None:
+    if hasattr(baseconversion, 'activate'):
         # This is pandas2ri if pandas is installed,
         # or numpy2ri otherwise
-        py2ri.activate()
+        baseconversion.activate()
 
     ip.register_magics(RMagics)
     # Initialising rpy2 interferes with readline. Since, at this point, we've
