@@ -16,18 +16,41 @@ import rpy2.rinterface as rinterface
 import rpy2.rlike.container as rlc
 
 from rpy2.robjects.robject import RObjectMixin, RObject
-from rpy2.robjects.vectors import *
+from rpy2.robjects.vectors import (BoolVector,
+                                   IntVector,
+                                   FloatVector,
+                                   ComplexVector,
+                                   StrVector,
+                                   FactorVector,
+                                   Vector,
+                                   ListVector,
+                                   DateVector,
+                                   POSIXct,
+                                   POSIXlt,
+                                   Array,
+                                   Matrix,
+                                   DataFrame)
 from rpy2.robjects.functions import Function, SignatureTranslatedFunction
 from rpy2.robjects.environments import Environment
 from rpy2.robjects.methods import RS4
 
 from . import conversion
 
-from rpy2.rinterface import Sexp, SexpVector, SexpClosure, SexpEnvironment, SexpS4, SexpExtPtr
+from rpy2.rinterface import (Sexp, 
+                             SexpVector, 
+                             SexpClosure, 
+                             SexpEnvironment, 
+                             SexpS4, 
+                             SexpExtPtr)
 _globalenv = rinterface.globalenv
 
 # missing values
-from rpy2.rinterface import NA_Real, NA_Integer, NA_Logical, NA_Character, NA_Complex, NULL
+from rpy2.rinterface import (NA_Real, 
+                             NA_Integer, 
+                             NA_Logical, 
+                             NA_Character, 
+                             NA_Complex, 
+                             NULL)
 
 if sys.version_info[0] == 2:
     py3str = unicode
@@ -51,7 +74,7 @@ def reval(string, envir = _globalenv):
     res = _reval(p, envir = envir)
     return res
 
-default_converter = conversion.make_converter()
+default_converter = conversion.make_converter('base empty converter')
 
 @default_converter.ri2ro.register(RObject)
 def _(obj):
@@ -96,6 +119,24 @@ def sexpvector_to_ro(obj):
 
 default_converter.ri2ro.register(SexpVector, sexpvector_to_ro)
 
+TYPEORDER = {bool: (0, BoolVector),
+             int: (1, IntVector),
+             float: (2, FloatVector),
+             complex: (3, ComplexVector),
+             str: (4, StrVector)}
+def sequence_to_vector(lst):
+    curr_typeorder = -1
+    for i, elt in enumerate(lst):
+        cls = type(elt)
+        if cls in TYPEORDER:
+            if TYPEORDER[cls][0] > curr_typeorder:
+                curr_typeorder, curr_type = TYPEORDER[cls]
+        else:
+            raise ValueError('The element %i in the list has a type that cannot be handled.' % i)
+    res = curr_type(lst)
+    return res
+
+
 @default_converter.ri2ro.register(SexpClosure)
 def _(obj):
     return SignatureTranslatedFunction(obj)
@@ -122,7 +163,8 @@ def _(obj):
 
 
 def default_py2ri(o):
-    """ Convert an arbitrary Python object to an :class:`rpy2.rinterface.Sexp` object.
+    """ Convert an arbitrary Python object to a
+    :class:`rpy2.rinterface.Sexp` object.
     Creates an R object with the content of the Python object,
     wich means data copying.
     :param o: object
@@ -272,7 +314,7 @@ class R(object):
 r = R()
 
 conversion.converter = default_converter
-conversion.ri2ro, conversion.py2ri, conversion.py2ro, conversion.ri2py = conversion.converter
+converter_name, conversion.ri2ro, conversion.py2ri, conversion.py2ro, conversion.ri2py, converter_lineage = conversion.converter
 
 default_ri2ro = conversion.converter.ri2ro
 default_py2ri = conversion.converter.py2ri
