@@ -1,13 +1,22 @@
 import os
 from types import ModuleType
+from collections import defaultdict
 from warnings import warn
 import rpy2.rinterface as rinterface
 import rpy2.robjects.lib
 from . import conversion
-from rpy2.robjects.functions import SignatureTranslatedFunction, docstring_property, DocumentedSTFunction
+from rpy2.robjects.functions import (SignatureTranslatedFunction, 
+                                     docstring_property, 
+                                     DocumentedSTFunction)
 from rpy2.robjects.constants import NULL
 from rpy2.robjects import Environment
-from rpy2.robjects.packages_utils import _libpaths, get_packagepath, _packages
+from rpy2.robjects.packages_utils import (_libpaths, 
+                                          get_packagepath, 
+                                          _packages,
+                                          default_symbol_r2python,
+                                          default_symbol_check_after,
+                                          _map_symbols,
+                                          _fix_map_symbols)
 import rpy2.robjects.help as rhelp
 
 _require = rinterface.baseenv['require']
@@ -140,7 +149,6 @@ class PackageData(object):
                  'envir': env})
         return Environment(env)
 
-
 class Package(ModuleType):
     """ Models an R package
     (and can do so from an arbitrary environment - with the caution
@@ -154,16 +162,29 @@ class Package(ModuleType):
     __fill_rpy2r__ = None
     __update_dict__ = None
     _exported_names = None
+    _symbol_r2python = None
     __version__ = None
     __rdata__ = None
 
     def __init__(self, env, name, translation = {}, 
                  exported_names = None, on_conflict = 'fail',
-                 version = None):
+                 version = None,
+                 symbol_r2python = default_symbol_r2python,
+                 symbol_check_after = default_symbol_check_after):
         """ Create a Python module-like object from an R environment,
         using the specified translation if defined. 
 
+        - env: R environment
+        - name: package name
+        - translation: `dict` with R names as keys and corresponding Python
+                       names as values
+        - exported_names: `set` of names/symbols to expose to instance users
         - on_conflict: 'fail' or 'warn' (default: 'fail')
+        - version: version string for the package
+        - symbol_r2python: function to convert R symbols into Python symbols.
+                           The default translate `.` into `_`.
+        - symbol_check_after: function to check the Python symbols obtained
+                              from `symbol_r2python`.
         """
 
         super(Package, self).__init__(name)
@@ -175,6 +196,8 @@ class Package(ModuleType):
         if exported_names is None:
             exported_names = set(self._env.keys())
         self._exported_names = exported_names
+        self._symbol_r2python = symbol_r2python
+        self._symbol_check_after = symbol_check_after
         self.__fill_rpy2r__(on_conflict = on_conflict)
         self._exported_names = self._exported_names.difference(mynames)
         self.__version__ = version
@@ -195,33 +218,34 @@ class Package(ModuleType):
         assert(on_conflict in ('fail', 'warn'))
 
         name = self.__rname__
-        for rname in self._env:
-            if rname in self._translation:
-                rpyname = self._translation[rname]
-            else:
-                dot_i = rname.find('.')
-                if dot_i > -1:
-                    rpyname = rname.replace('.', '_')
-                    if rpyname in self._rpy2r:
-                        msg = ('Conflict when converting R symbol'+\
-                                   ' in the package "%s"' +\
-                                   ' to a Python symbol ' +\
-                                   '(%s -> %s while there is already'+\
-                                   ' %s)') %(self.__rname__,
-                                             rname, rpyname,
-                                             rpyname)
-                        if on_conflict == 'fail':
-                            raise LibraryError(msg)
-                        else:
-                            warn(msg)
-                            continue
-                else:
-                    rpyname = rname
-                if rpyname in self.__dict__ or rpyname == '__dict__':
-                    raise LibraryError('The symbol ' + rname +\
-                                       ' in the package "' + name + '"' +\
-                                       ' is conflicting with ' +\
-                                       'a Python object attribute')
+
+        (symbol_mapping, 
+         conflicts, 
+         resolutions) = _map_symbols(self._env,
+                                     translation = self._translation,
+                                     symbol_r2python = self._symbol_r2python,
+                                     symbol_check_after = self._symbol_check_after)
+        msg_prefix = 'Conflict when converting R symbols'+\
+                     ' in the package "%s"' % self.__rname__ +\
+                     ' to Python symbols: \n-'
+        exception = LibraryError
+        _fix_map_symbols(symbol_mapping,
+                         conflicts,
+                         on_conflict,
+                         msg_prefix,
+                         exception)
+        symbol_mapping.update(resolutions)
+        reserved_pynames = set(dir(self))
+        for rpyname, rnames in symbol_mapping.items():
+            # last paranoid check
+            if len(rnames) > 1:
+                raise ValueError('Only one R name should be associated with %s (and we have %s)' % (rpyname, str(rnames)))
+            rname = rnames[0]
+            if rpyname in reserved_pynames:
+                raise LibraryError('The symbol ' + rname +\
+                                   ' in the package "' + name + '"' +\
+                                   ' is conflicting with' +\
+                                   ' a Python object attribute')
             self._rpy2r[rpyname] = rname
             if (rpyname != rname) and (rname in self._exported_names):
                 self._exported_names.remove(rname)
@@ -236,10 +260,9 @@ class Package(ModuleType):
             #FIXME: shouldn't the original R name be also in the __dict__ ?
             self.__dict__[rpyname] = rpyobj
 
-
     def __repr__(self):
         s = super(Package, self).__repr__()
-        return 'rpy2.robjecs.packages.Package as a ' + s
+        return 'rpy2.robjects.packages.Package as a ' + s
 
 
 
@@ -251,7 +274,10 @@ class SignatureTranslatedPackage(Package):
         super(SignatureTranslatedPackage, self).__fill_rpy2r__(on_conflict = on_conflict)
         for name, robj in self.__dict__.items():
             if isinstance(robj, rinterface.Sexp) and robj.typeof == rinterface.CLOSXP:
-                self.__dict__[name] = SignatureTranslatedFunction(self.__dict__[name])
+                self.__dict__[name] = SignatureTranslatedFunction(self.__dict__[name],
+                                                                  on_conflict = on_conflict,
+                                                                  symbol_r2python = self._symbol_r2python,
+                                                                  symbol_check_after = self._symbol_check_after)
                 
 
 class SignatureTranslatedAnonymousPackage(SignatureTranslatedPackage):
@@ -358,6 +384,8 @@ def importr(name,
             signature_translation = True,
             suppress_messages = True,
             on_conflict = 'fail',
+            symbol_r2python = default_symbol_r2python,
+            symbol_check_after = default_symbol_check_after,
             data = True):
     """ Import an R package.
 
@@ -376,6 +404,11 @@ def importr(name,
 
     - on_conflict: 'fail' or 'warn' (default: 'fail')
 
+    - symbol_r2python: function to translate R symbols into Python symbols
+
+    - symbol_check_after: function to check the Python symbol obtained
+                          from `symbol_r2python`.
+
     - data: embed a PackageData objects under the attribute 
       name __rdata__ (default: True)
 
@@ -409,12 +442,16 @@ def importr(name,
                                   translation = robject_translations,
                                   exported_names = exported_names,
                                   on_conflict = on_conflict,
-                                  version = version)
+                                  version = version,
+                                  symbol_r2python = symbol_r2python,
+                                  symbol_check_after = symbol_check_after)
     else:
         pack = InstalledPackage(env, name, translation = robject_translations,
                                 exported_names = exported_names,
                                 on_conflict = on_conflict,
-                                version = version)
+                                version = version,
+                                symbol_r2python = symbol_r2python,
+                                symbol_check_after = symbol_check_after)
     if data:
         if pack.__rdata__ is not None:
             warn('While importing the R package "%s", the rpy2 Package object is masking a translated R symbol "__rdata__" already present' % name)
